/**
 * SrbnFreaks(SOROBAN FREAKS)
 * (c) Kenta Cho (cs8k-cyu@asahi-net.or.jp)
 * http://www.asahi-net.or.jp/~cs8k-cyu/
 *
 * drawsoroban.c(draw soroban status)
 */

static void checkTotal();

// soroban parameters
#define SOROBAN_LINE_NMB 4
#define DAMA_WIDTH 28
#define DAMA_HEIGHT 16
#define DAMA_HALF_HEIGHT 8

// soroban line state handler
SorobanLine sl[SOROBAN_LINE_NMB];

static void initSoroban() {
  int i;
  for ( i=0 ; i<SOROBAN_LINE_NMB ; i++ ) {
    sl[i].x = i*40 + 10;
    sl[i].topy = 24; sl[i].btmy = 60;
    sl[i].topIdx = sl[i].btmIdx = 0;
    sl[i].idTopIdx = 1;
  }
}

static void clearSoroban() {
  RectangleType rct;
  rct.topLeft.x = 0;
  rct.topLeft.y = 16;
  rct.extent.x = 160;
  rct.extent.y = 160-16;
  WinEraseRectangle(&rct, 0);
}

static void drawTickLine(int x1, int y1, int x2, int y2) {
  WinDrawLine(x1, y1, x2, y2);
  WinDrawLine(x1, y1+1, x2, y2+1);
}

static void drawTickLine2(int x1, int y1, int x2, int y2) {
  WinDrawLine(x1, y1, x2, y2);
  WinDrawLine(x1+1, y1, x2+1, y2);
}

// bitmap indexs
#define SRDAMA_IDX 0
#define SRSEN_IDX 1

static void drawSrDama(int x, int y) {
  WinDrawBitmap(objectBitmapPtr[SRDAMA_IDX], x, y);
}

static void drawSrSen(int x, int y) {
  WinDrawBitmap(objectBitmapPtr[SRSEN_IDX], x, y);
  WinDrawBitmap(objectBitmapPtr[SRSEN_IDX], x, y+DAMA_HALF_HEIGHT);
}

/**
 * draw up arrow indicator
 */
static void drawUpArrow(int x, int y) {
  drawTickLine(x-DAMA_WIDTH/3, y, x+DAMA_WIDTH/3, y);
  drawTickLine2(x-DAMA_WIDTH/3, y, x-DAMA_WIDTH/3, y-DAMA_HEIGHT); 
  drawTickLine(x-DAMA_WIDTH/3, y-DAMA_HEIGHT,
	       x-DAMA_WIDTH/3+5, y-DAMA_HEIGHT+5);
}

/**
 * draw down arrow indicator
 */
static void drawDownArrow(int x, int y) {
  drawTickLine(x-DAMA_WIDTH/3, y, x+DAMA_WIDTH/3, y);
  drawTickLine2(x-DAMA_WIDTH/3, y, x-DAMA_WIDTH/3, y+DAMA_HEIGHT); 
  drawTickLine(x-DAMA_WIDTH/3, y+DAMA_HEIGHT,
	       x-DAMA_WIDTH/3+5, y+DAMA_HEIGHT-5);
}

Boolean indicatorOn;

/**
 * draw arrow indicator for line[idx] on top
 */
static void drawTopLineIndicator(int idx) {
  SorobanLine stl = sl[idx];
  if ( !indicatorOn ) return;
  if ( stl.topIdx != stl.idTopIdx ) {
    if ( stl.idTopIdx == 0 ) {
      drawDownArrow(stl.x, stl.topy);
    } else {
      drawUpArrow(stl.x, stl.topy+DAMA_HEIGHT*2-3); 
    }
  }
}

/**
 * draw arrow indicator for line[idx] on bottom
 */
static void drawBtmLineIndicator(int idx) {
  SorobanLine stl = sl[idx];
  if ( !indicatorOn ) return;
  if ( stl.btmIdx > stl.idBtmIdx ) {
    drawDownArrow(stl.x, stl.btmy+stl.idBtmIdx*DAMA_HEIGHT);
  } else if ( stl.btmIdx < stl.idBtmIdx ) {    
    drawUpArrow(stl.x, stl.btmy+stl.idBtmIdx*DAMA_HEIGHT+DAMA_HEIGHT-3);
  }
}

/**
 * repaint top
 */
static void repaintTopDama(int idx) {
  SorobanLine sbl = sl[idx];
  RectangleType rct;
  rct.extent.x = sl[1].x-sl[0].x;
  rct.topLeft.x = sbl.x+DAMA_WIDTH-rct.extent.x;
  rct.topLeft.y = sbl.topy;
  rct.extent.y = DAMA_HEIGHT*2;
  WinEraseRectangle(&rct, 0);
  if ( sbl.topIdx == 0 ) {
    drawSrSen(sbl.x, sbl.topy);
    drawSrDama(sbl.x, sbl.topy+DAMA_HEIGHT);
  } else {
    drawSrDama(sbl.x, sbl.topy);
    drawSrSen(sbl.x, sbl.topy+DAMA_HEIGHT);
  }
}

/**
 * repaint bottom
 */
static void repaintBtmDama(int idx) {
  int i;
  SorobanLine sbl = sl[idx];
  RectangleType rct;
  rct.extent.x = sl[1].x-sl[0].x;
  rct.topLeft.x = sbl.x+DAMA_WIDTH-rct.extent.x;
  rct.topLeft.y = sbl.btmy;
  rct.extent.y = DAMA_HEIGHT*5;
  WinEraseRectangle(&rct, 0);
  for ( i=0 ; i<sbl.btmIdx ; i++ ) {
    drawSrDama(sbl.x, sbl.btmy+i*DAMA_HEIGHT);
  }
  drawSrSen(sbl.x, sbl.btmy+i*DAMA_HEIGHT);
  i++;
  for ( ; i<5 ; i++ ) {
    drawSrDama(sbl.x, sbl.btmy+i*DAMA_HEIGHT);
  }
}

static void repaintSoroban() {
  int i, j;
  RectangleType rct;
  int ly1 = sl[0].topy+DAMA_HEIGHT*2, ly2 = sl[0].btmy-1;
  for ( i=0 ; i<SOROBAN_LINE_NMB ; i++ ) {
    repaintTopDama(i);
    repaintBtmDama(i);
  }
  WinDrawLine(0, ly1, 159, ly1);
  WinDrawLine(0, ly2, 159, ly2);
  rct.topLeft.x = 0;
  rct.topLeft.y = sl[0].topy-5;
  rct.extent.x = 160;
  rct.extent.y = 4;
  WinDrawRectangle(&rct, 0);
  rct.topLeft.y = sl[0].btmy+DAMA_HEIGHT*5+1;
  WinDrawRectangle(&rct, 0);
}

static Boolean moveTopDama(int lineIdx, int damaIdx, int upOrDown) {
  if ( damaIdx<0 || damaIdx>=2 ) return FALSE;
  if ( upOrDown == 0 ) {
    if ( sl[lineIdx].topIdx >= damaIdx ) return FALSE;
    sl[lineIdx].topIdx = 1;
  } else {
    if ( sl[lineIdx].topIdx <= damaIdx ) return FALSE;
    sl[lineIdx].topIdx = 0;
  }
  repaintTopDama(lineIdx);
  drawTopLineIndicator(lineIdx);
  checkTotal();
  return TRUE;
}

static Boolean moveBtmDama(int lineIdx, int damaIdx, int upOrDown) {
  if ( damaIdx<0 || damaIdx>=5 ) return FALSE;
  if ( upOrDown == 0 ) {
    if ( sl[lineIdx].btmIdx >= damaIdx ) return FALSE;
    sl[lineIdx].btmIdx = damaIdx;
  } else {
    if ( sl[lineIdx].btmIdx <= damaIdx ) return FALSE;
    sl[lineIdx].btmIdx = damaIdx;
  }
  repaintBtmDama(lineIdx);
  drawBtmLineIndicator(lineIdx);
  checkTotal();
  return TRUE;
}

static void clearStartIndicator() {
  int i;
  for ( i=0 ; i<SOROBAN_LINE_NMB ; i++ ) {
    sl[i].topIdx = 1;
    repaintTopDama(i);
  }
}

/**
 * draw negaimasitewa indicator
 */
static void drawStartIndicator() {
  int y = sl[0].topy+DAMA_HEIGHT*2-DAMA_HALF_HEIGHT/2;
  WinEraseLine(0, y-1, 156, y-1);
  WinEraseLine(0, y+2, 156, y+2);
  WinEraseLine(145, y-11-3, 156, y-3);
  WinEraseLine(145, y-11, 156, y);
  WinEraseLine(145, y-11-3, 145, y-11);
  drawTickLine(0, y, 156, y);
  drawTickLine(146, y-10-2, 156, y-2);
}

/**
 * draw indicator that represents total
 */
static void drawIndicator() {
  int i, tt = total, an;
  for ( i=SOROBAN_LINE_NMB-1 ; i>=0 ; i--, tt/=10 ) {
    an = tt%10;
    if ( an >= 5 ) {
      sl[i].idTopIdx = 0;
      an -= 5;
    } else {
      sl[i].idTopIdx = 1;
    }
    sl[i].idBtmIdx = an;
    drawTopLineIndicator(i);
    drawBtmLineIndicator(i);
  }
}
